@@ -41,6 +41,9 @@ static unsigned long set_mult     = 0, get_mult     = 0, mult     = 0;
 #ifdef HDIO_GET_KEEPSETTINGS
 static unsigned long set_keep     = 0, get_keep     = 0, keep     = 0;
 #endif
+#ifdef HDIO_SET_CHIPSET
+static unsigned long set_chipset  = 0, get_chipset  = 0, chipset  = 0;
+#endif
 #ifdef HDIO_DRIVE_CMD
 static unsigned long set_dkeep    = 0, get_dkeep    = 0, dkeep    = 0;
 static unsigned long set_standby  = 0, get_standby  = 0, standby  = 0;
@@ -156,8 +159,12 @@ void flush_buffer_cache (int fd)
 	if (ioctl(fd, BLKFLSBUF, NULL))		/* do it again, big time */
 		perror("BLKFLSBUF failed");
 #ifdef HDIO_DRIVE_CMD
-	if (ioctl(fd, HDIO_DRIVE_CMD, NULL))	/* wait for completion */
-		perror("HDIO_DRIVE_CMD failed");
+	if (is_scsi_hd) {
+		sleep(1);
+	} else {
+		if (ioctl(fd, HDIO_DRIVE_CMD, NULL))	/* await completion */
+			perror("HDIO_DRIVE_CMD failed");
+	}
 #endif
 }
 
@@ -419,6 +426,15 @@ void process_dev (char *devname)
 		if (ioctl(fd, BLKRASET, readahead)) 
 			perror(" BLKRASET failed");
 	}
+#ifdef HDIO_SET_CHIPSET
+	if (set_chipset) {
+		no_scsi();
+		if (get_chipset)
+			printf(" setting chipset to %ld\n", chipset);
+		if (ioctl(fd, HDIO_SET_CHIPSET, chipset)) 
+			perror(" HDIO_SET_CHIPSET failed");
+	}
+#endif
 	if (set_mult) {
 		no_scsi();
 		if (get_mult)
@@ -568,6 +584,16 @@ void process_dev (char *devname)
 			on_off(parm);
 		}
 	}
+#ifdef HDIO_GET_CHIPSET
+	if (get_chipset) {
+		no_scsi();
+		if (ioctl(fd, HDIO_GET_CHIPSET, &parm))
+			perror(" HDIO_GET_CHIPSET failed");
+		else {
+			printf(" chipset   =%3ld\n", parm);
+		}
+	}
+#endif
 	if ((verbose && !is_scsi_hd) || get_unmask) {
 		no_scsi();
 		if (ioctl(fd, HDIO_GET_UNMASKINTR, &parm))
@@ -621,12 +647,15 @@ void process_dev (char *devname)
 
 void usage_error (void)
 {
-	fprintf(stderr,"%s - get/set hard disk parameters - version 2.2\n\n", progname);
+	fprintf(stderr,"%s - get/set hard disk parameters - version 2.3\n\n", progname);
 	fprintf(stderr,"Usage:  %s  [options] [device] ..\n\n", progname);
 	fprintf(stderr,"Options:\n");
 	fprintf(stderr," -a    get/set fs readahead\n");
 #ifdef HDIO_DRIVE_CMD
 	fprintf(stderr," -A  * set drive read-lookahead flag (0/1)\n");
+#endif
+#ifdef HDIO_SET_CHIPSET
+	fprintf(stderr," -c    get/set IDE chipset/interface type\n");
 #endif
 	fprintf(stderr," -f    flush buffer cache for device on exit\n");
 	fprintf(stderr," -g    display drive geometry\n");
@@ -727,6 +756,13 @@ int main(int argc, char **argv)
 						noisy = 1;
 						GET_NUMBER(set_mult,mult);
 						break;
+#ifdef HDIO_SET_CHIPSET
+					case 'c':
+						get_chipset = noisy;
+						noisy = 1;
+						GET_NUMBER(set_chipset,chipset);
+						break;
+#endif /* HDIO_SET_CHIPSET */
 #ifdef HDIO_DRIVE_CMD
 					case 'S':
 						get_standby = noisy;
